package org.jbpm.examples.evaluation;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.drools.process.instance.WorkItemHandler;
import org.drools.runtime.process.WorkItem;
import org.drools.runtime.process.WorkItemManager;
import org.jbpm.task.AccessType;
import org.jbpm.task.Group;
import org.jbpm.task.I18NText;
import org.jbpm.task.OrganizationalEntity;
import org.jbpm.task.PeopleAssignments;
import org.jbpm.task.SubTasksStrategy;
import org.jbpm.task.SubTasksStrategyFactory;
import org.jbpm.task.Task;
import org.jbpm.task.TaskData;
import org.jbpm.task.User;
import org.jbpm.task.event.TaskClaimedEvent;
import org.jbpm.task.event.TaskCompletedEvent;
import org.jbpm.task.event.TaskEventListener;
import org.jbpm.task.event.TaskFailedEvent;
import org.jbpm.task.event.TaskSkippedEvent;
import org.jbpm.task.service.ContentData;
import org.jbpm.task.service.Operation;
import org.jbpm.task.service.TaskService;
import org.jbpm.task.service.TaskServiceSession;

public class LocalHumanTaskHandler implements WorkItemHandler {

	private WorkItemManager manager = null;
	private TaskServiceSession taskSession;
	private TaskService taskService;

	public LocalHumanTaskHandler(TaskService taskService) throws IllegalArgumentException, IOException
	{
		super();
		if(taskService == null)
		{
			throw new IllegalArgumentException("Null task service provided");
		}
		this.taskService = taskService;		
		this.taskSession = taskService.createSession();
		// Add users here...
		Map vars = new HashMap();
		Reader reader = new InputStreamReader( DemoTaskService.class.getResourceAsStream( "LoadUsers.mvel" ) );     
		@SuppressWarnings("unchecked")
		Map<String, User> users = ( Map<String, User> ) eval( reader, vars );   
		for ( User user : users.values() ) {
			taskSession.addUser( user );
			System.out.println(user + " is added into task session");
		}           
		reader = new InputStreamReader( DemoTaskService.class.getResourceAsStream( "LoadGroups.mvel" ) );      
		Map<String, Group> groups = ( Map<String, Group> ) eval( reader, vars );     
		for ( Group group : groups.values() ) {
			taskSession.addGroup( group );
			System.out.println(group + " is added into task session");
		}
	}

	public void connect()
	{
		TaskEventHandler eventResponseHandler = new TaskEventHandler(manager, taskSession);
		this.taskService.addEventListener(eventResponseHandler);
	}

	public void setClient(TaskServiceSession taskSession) {
		this.taskSession = taskSession;
	}

	public void setManager(WorkItemManager manager) {
		this.manager = manager;
	}

	public void executeWorkItem(WorkItem workItem, WorkItemManager manager) {
		if (this.manager == null) {
			this.manager = manager;
		} else {
			if (this.manager != manager) {
				throw new IllegalArgumentException(
				"This WSHumanTaskHandler can only be used for one WorkItemManager");
			}
		}
		connect();
		Task task = new Task();
		String taskName = (String) workItem.getParameter("TaskName");
		if (taskName != null) {
			List<I18NText> names = new ArrayList<I18NText>();
			names.add(new I18NText("en-UK", taskName));
			task.setNames(names);
		}
		String comment = (String) workItem.getParameter("Comment");
		if (comment != null) {
			List<I18NText> descriptions = new ArrayList<I18NText>();
			descriptions.add(new I18NText("en-UK", comment));
			task.setDescriptions(descriptions);
			List<I18NText> subjects = new ArrayList<I18NText>();
			subjects.add(new I18NText("en-UK", comment));
			task.setSubjects(subjects);
		}
		String priorityString = (String) workItem.getParameter("Priority");
		int priority = 0;
		if (priorityString != null) {
			try {
				priority = new Integer(priorityString);
			} catch (NumberFormatException e) {
				// do nothing
			}
		}
		task.setPriority(priority);

		TaskData taskData = new TaskData();
		taskData.setWorkItemId(workItem.getId());
		taskData.setProcessInstanceId(workItem.getProcessInstanceId());
		taskData.setSkipable(!"false".equals(workItem.getParameter("Skippable")));
		//Sub Task Data
		Long parentId = (Long) workItem.getParameter("ParentId");
		if(parentId != null){
			taskData.setParentId(parentId);
		}

		String subTaskStrategiesCommaSeparated = (String) workItem.getParameter("SubTaskStrategies");
		if(subTaskStrategiesCommaSeparated!= null && !subTaskStrategiesCommaSeparated.equals("")){
			String[] subTaskStrategies =  subTaskStrategiesCommaSeparated.split(",");
			List<SubTasksStrategy> strategies = new ArrayList<SubTasksStrategy>();
			for(String subTaskStrategyString : subTaskStrategies){
				SubTasksStrategy subTaskStrategy = SubTasksStrategyFactory.newStrategy(subTaskStrategyString);
				strategies.add(subTaskStrategy);
			}
			task.setSubTaskStrategies(strategies);
		}

		PeopleAssignments assignments = new PeopleAssignments();
		List<OrganizationalEntity> potentialOwners = new ArrayList<OrganizationalEntity>();

		String actorId = (String) workItem.getParameter("ActorId");
		if (actorId != null && actorId.trim().length() > 0) {
			String[] actorIds = actorId.split(",");
			for (String id: actorIds) {
				potentialOwners.add(new User(id.trim()));
			}
			//Set the first user as creator ID??? hmmm might be wrong
			if (potentialOwners.size() > 0){
				taskData.setCreatedBy((User)potentialOwners.get(0));
			}
		}

		String groupId = (String) workItem.getParameter("GroupId");
		if (groupId != null && groupId.trim().length() > 0) {
			String[] groupIds = groupId.split(",");
			for (String id: groupIds) {
				potentialOwners.add(new Group(id.trim()));
			}
		}

		assignments.setPotentialOwners(potentialOwners);
		List<OrganizationalEntity> businessAdministrators = new ArrayList<OrganizationalEntity>();
		businessAdministrators.add(new User("Administrator"));
		assignments.setBusinessAdministrators(businessAdministrators);
		task.setPeopleAssignments(assignments);

		task.setTaskData(taskData);

		ContentData content = null;
		Object contentObject = workItem.getParameter("Content");
		if (contentObject == null) {
			contentObject = workItem.getParameters();
		}
		if (contentObject != null) {
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			ObjectOutputStream out;
			try {
				out = new ObjectOutputStream(bos);
				out.writeObject(contentObject);
				out.close();
				content = new ContentData();
				content.setContent(bos.toByteArray());
				content.setAccessType(AccessType.Inline);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		taskSession.addTask(task, content);
	}

	public void dispose() throws Exception {
		if (taskSession != null) {
			taskSession.dispose();
		}
	}

	public void abortWorkItem(WorkItem workItem, WorkItemManager manager) {
		Task task = taskSession.getTaskByWorkItemId(workItem.getId());
		if (task != null) {
			taskSession.taskOperation(Operation.Skip, task.getId(), "Administrator", null, null, null);
		}		
	}

	public TaskServiceSession getTaskSession() {
		return taskSession;
	}

	private static class TaskEventHandler implements TaskEventListener
	{

		private WorkItemManager manager;
		private TaskServiceSession taskSession;

		public TaskEventHandler(WorkItemManager manager, TaskServiceSession taskSession) {
			super();
			this.manager = manager;
			this.taskSession = taskSession;
		}

		public void taskClaimed(TaskClaimedEvent event) {
			// TODO Auto-generated method stub
		}

		public void taskCompleted(TaskCompletedEvent event) {
			long taskId = event.getTaskId();			
			Task task = taskSession.getTask(taskId);
			long workItemId = task.getTaskData().getWorkItemId();
			String userId = task.getTaskData().getActualOwner().getId();
			Map<String, Object> results = new HashMap<String, Object>();
			results.put("ActorId", userId);
			long contentId = task.getTaskData().getOutputContentId();
			if (contentId != -1) {
				ByteArrayInputStream bis = new ByteArrayInputStream(taskSession.getContent(contentId).getContent());
				ObjectInputStream in;
				try {
					in = new ObjectInputStream(bis);
					Object result = in.readObject();
					in.close();
					results.put("Result", result);
					if (result instanceof Map) {
						Map<?, ?> map = (Map<?, ?>) result;
						for (Map.Entry<?, ?> entry: map.entrySet()) {
							if (entry.getKey() instanceof String) {
								results.put((String) entry.getKey(), entry.getValue());
							}
						}
					}
					manager.completeWorkItem(task.getTaskData().getWorkItemId(), results);
				} catch (IOException e) {
					e.printStackTrace();
				} catch (ClassNotFoundException e) {
					e.printStackTrace();
				}

			} else {
				manager.completeWorkItem(workItemId, results);
			}
		}

		public void taskFailed(TaskFailedEvent event) {
			long taskId = event.getTaskId();			
			Task task = taskSession.getTask(taskId);
			long workItemId = task.getTaskData().getWorkItemId();
			manager.abortWorkItem(workItemId);

		}

		public void taskSkipped(TaskSkippedEvent event) {
			long taskId = event.getTaskId();			
			Task task = taskSession.getTask(taskId);
			long workItemId = task.getTaskData().getWorkItemId();
			manager.abortWorkItem(workItemId);
		}
	}
}